# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Literal, Required, Annotated, TypedDict

from ...._utils import PropertyInfo
from .action_common_options_param import ActionCommonOptionsParam

__all__ = ["ActionPressKeyParams"]

KeysType = List[
    Literal[
        "a",
        "b",
        "c",
        "d",
        "e",
        "f",
        "g",
        "h",
        "i",
        "j",
        "k",
        "l",
        "m",
        "n",
        "o",
        "p",
        "q",
        "r",
        "s",
        "t",
        "u",
        "v",
        "w",
        "x",
        "y",
        "z",
        "0",
        "1",
        "2",
        "3",
        "4",
        "5",
        "6",
        "7",
        "8",
        "9",
        "f1",
        "f2",
        "f3",
        "f4",
        "f5",
        "f6",
        "f7",
        "f8",
        "f9",
        "f10",
        "f11",
        "f12",
        "control",
        "alt",
        "shift",
        "meta",
        "win",
        "cmd",
        "option",
        "arrowUp",
        "arrowDown",
        "arrowLeft",
        "arrowRight",
        "home",
        "end",
        "pageUp",
        "pageDown",
        "enter",
        "space",
        "tab",
        "escape",
        "backspace",
        "delete",
        "insert",
        "capsLock",
        "numLock",
        "scrollLock",
        "pause",
        "printScreen",
        ";",
        "=",
        ",",
        "-",
        ".",
        "/",
        "`",
        "[",
        "\\",
        "]",
        "'",
        "numpad0",
        "numpad1",
        "numpad2",
        "numpad3",
        "numpad4",
        "numpad5",
        "numpad6",
        "numpad7",
        "numpad8",
        "numpad9",
        "numpadAdd",
        "numpadSubtract",
        "numpadMultiply",
        "numpadDivide",
        "numpadDecimal",
        "numpadEnter",
        "numpadEqual",
        "volumeUp",
        "volumeDown",
        "volumeMute",
        "mediaPlayPause",
        "mediaStop",
        "mediaNextTrack",
        "mediaPreviousTrack",
    ]
]


class ActionPressKeyParams(TypedDict, total=False):
    keys: Required[KeysType]
    """This is an array of keyboard keys to press.

    Supports cross-platform compatibility.
    """

    combination: bool
    """Whether to press keys as combination (simultaneously) or sequentially.

    When true, all keys are pressed together as a shortcut (e.g., Ctrl+C). When
    false, keys are pressed one by one in sequence.
    """

    include_screenshot: Annotated[bool, PropertyInfo(alias="includeScreenshot")]
    """⚠️ DEPRECATED: Use `options.screenshot.phases` instead.

    This field will be ignored when `options.screenshot` is provided. Whether to
    include screenshots in the action response. If false, the screenshot object will
    still be returned but with empty URIs. Default is false.
    """

    options: ActionCommonOptionsParam
    """Action common options"""

    output_format: Annotated[Literal["base64", "storageKey"], PropertyInfo(alias="outputFormat")]
    """⚠️ DEPRECATED: Use `options.screenshot.outputFormat` instead.

    Type of the URI. default is base64. This field will be ignored when
    `options.screenshot` is provided.
    """

    presigned_expires_in: Annotated[str, PropertyInfo(alias="presignedExpiresIn")]
    """⚠️ DEPRECATED: Use `options.screenshot.presignedExpiresIn` instead.

    Presigned url expires in. Only takes effect when outputFormat is storageKey.
    This field will be ignored when `options.screenshot` is provided.

    Supported time units: ms (milliseconds), s (seconds), m (minutes), h (hours)
    Example formats: "500ms", "30s", "5m", "1h" Default: 30m
    """

    screenshot_delay: Annotated[str, PropertyInfo(alias="screenshotDelay")]
    """⚠️ DEPRECATED: Use `options.screenshot.delay` instead.

    This field will be ignored when `options.screenshot` is provided.

    Delay after performing the action, before taking the final screenshot.

    Execution flow:

    1. Take screenshot before action
    2. Perform the action
    3. Wait for screenshotDelay (this parameter)
    4. Take screenshot after action

    Example: '500ms' means wait 500ms after the action before capturing the final
    screenshot.

    Supported time units: ms (milliseconds), s (seconds), m (minutes), h (hours)
    Example formats: "500ms", "30s", "5m", "1h" Default: 500ms Maximum allowed: 30s
    """



from __future__ import annotations

import os
import re
import inspect
import functools
from typing import (
    Any,
    Tuple,
    Mapping,
    TypeVar,
    Callable,
    Iterable,
    Sequence,
    cast,
    overload,
)
from pathlib import Path
from datetime import date, datetime
from typing_extensions import TypeGuard

import sniffio

from .._types import NotGiven, FileTypes, NotGivenOr, HeadersLike

_T = TypeVar("_T")
_TupleT = TypeVar("_TupleT", bound=Tuple[object, ...])
_MappingT = TypeVar("_MappingT", bound=Mapping[str, object])
_SequenceT = TypeVar("_SequenceT", bound=Sequence[object])
CallableT = TypeVar("CallableT", bound=Callable[..., Any])


def flatten(t: Iterable[Iterable[_T]]) -> list[_T]:
    return [item for sublist in t for item in sublist]


def extract_files(
    # TODO: this needs to take Dict but variance issues.....
    # create protocol type ?
    query: Mapping[str, object],
    *,
    paths: Sequence[Sequence[str]],
) -> list[tuple[str, FileTypes]]:
    """Recursively extract files from the given dictionary based on specified paths.

    A path may look like this ['foo', 'files', '<array>', 'data'].

    Note: this mutates the given dictionary.
    """
    files: list[tuple[str, FileTypes]] = []
    for path in paths:
        files.extend(_extract_items(query, path, index=0, flattened_key=None))
    return files


def _extract_items(
    obj: object,
    path: Sequence[str],
    *,
    index: int,
    flattened_key: str | None,
) -> list[tuple[str, FileTypes]]:
    try:
        key = path[index]
    except IndexError:
        if isinstance(obj, NotGiven):
            # no value was provided - we can safely ignore
            return []

        # cyclical import
        from .._files import assert_is_file_content

        # We have exhausted the path, return the entry we found.
        assert flattened_key is not None

        if is_list(obj):
            files: list[tuple[str, FileTypes]] = []
            for entry in obj:
                assert_is_file_content(entry, key=flattened_key + "[]" if flattened_key else "")
                files.append((flattened_key + "[]", cast(FileTypes, entry)))
            return files

        assert_is_file_content(obj, key=flattened_key)
        return [(flattened_key, cast(FileTypes, obj))]

    index += 1
    if is_dict(obj):
        try:
            # We are at the last entry in the path so we must remove the field
            if (len(path)) == index:
                item = obj.pop(key)
            else:
                item = obj[key]
        except KeyError:
            # Key was not present in the dictionary, this is not indicative of an error
            # as the given path may not point to a required field. We also do not want
            # to enforce required fields as the API may differ from the spec in some cases.
            return []
        if flattened_key is None:
            flattened_key = key
        else:
            flattened_key += f"[{key}]"
        return _extract_items(
            item,
            path,
            index=index,
            flattened_key=flattened_key,
        )
    elif is_list(obj):
        if key != "<array>":
            return []

        return flatten(
            [
                _extract_items(
                    item,
                    path,
                    index=index,
                    flattened_key=flattened_key + "[]" if flattened_key is not None else "[]",
                )
                for item in obj
            ]
        )

    # Something unexpected was passed, just ignore it.
    return []


def is_given(obj: NotGivenOr[_T]) -> TypeGuard[_T]:
    return not isinstance(obj, NotGiven)


# Type safe methods for narrowing types with TypeVars.
# The default narrowing for isinstance(obj, dict) is dict[unknown, unknown],
# however this cause Pyright to rightfully report errors. As we know we don't
# care about the contained types we can safely use `object` in it's place.
#
# There are two separate functions defined, `is_*` and `is_*_t` for different use cases.
# `is_*` is for when you're dealing with an unknown input
# `is_*_t` is for when you're narrowing a known union type to a specific subset


def is_tuple(obj: object) -> TypeGuard[tuple[object, ...]]:
    return isinstance(obj, tuple)


def is_tuple_t(obj: _TupleT | object) -> TypeGuard[_TupleT]:
    return isinstance(obj, tuple)


def is_sequence(obj: object) -> TypeGuard[Sequence[object]]:
    return isinstance(obj, Sequence)


def is_sequence_t(obj: _SequenceT | object) -> TypeGuard[_SequenceT]:
    return isinstance(obj, Sequence)


def is_mapping(obj: object) -> TypeGuard[Mapping[str, object]]:
    return isinstance(obj, Mapping)


def is_mapping_t(obj: _MappingT | object) -> TypeGuard[_MappingT]:
    return isinstance(obj, Mapping)


def is_dict(obj: object) -> TypeGuard[dict[object, object]]:
    return isinstance(obj, dict)


def is_list(obj: object) -> TypeGuard[list[object]]:
    return isinstance(obj, list)


def is_iterable(obj: object) -> TypeGuard[Iterable[object]]:
    return isinstance(obj, Iterable)


def deepcopy_minimal(item: _T) -> _T:
    """Minimal reimplementation of copy.deepcopy() that will only copy certain object types:

    - mappings, e.g. `dict`
    - list

    This is done for performance reasons.
    """
    if is_mapping(item):
        return cast(_T, {k: deepcopy_minimal(v) for k, v in item.items()})
    if is_list(item):
        return cast(_T, [deepcopy_minimal(entry) for entry in item])
    return item


# copied from https://github.com/Rapptz/RoboDanny
def human_join(seq: Sequence[str], *, delim: str = ", ", final: str = "or") -> str:
    size = len(seq)
    if size == 0:
        return ""

    if size == 1:
        return seq[0]

    if size == 2:
        return f"{seq[0]} {final} {seq[1]}"

    return delim.join(seq[:-1]) + f" {final} {seq[-1]}"


def quote(string: str) -> str:
    """Add single quotation marks around the given string. Does *not* do any escaping."""
    return f"'{string}'"


def required_args(*variants: Sequence[str]) -> Callable[[CallableT], CallableT]:
    """Decorator to enforce a given set of arguments or variants of arguments are passed to the decorated function.

    Useful for enforcing runtime validation of overloaded functions.

    Example usage:
    ```py
    @overload
    def foo(*, a: str) -> str: ...


    @overload
    def foo(*, b: bool) -> str: ...


    # This enforces the same constraints that a static type checker would
    # i.e. that either a or b must be passed to the function
    @required_args(["a"], ["b"])
    def foo(*, a: str | None = None, b: bool | None = None) -> str: ...
    ```
    """

    def inner(func: CallableT) -> CallableT:
        params = inspect.signature(func).parameters
        positional = [
            name
            for name, param in params.items()
            if param.kind
            in {
                param.POSITIONAL_ONLY,
                param.POSITIONAL_OR_KEYWORD,
            }
        ]

        @functools.wraps(func)
        def wrapper(*args: object, **kwargs: object) -> object:
            given_params: set[str] = set()
            for i, _ in enumerate(args):
                try:
                    given_params.add(positional[i])
                except IndexError:
                    raise TypeError(
                        f"{func.__name__}() takes {len(positional)} argument(s) but {len(args)} were given"
                    ) from None

            for key in kwargs.keys():
                given_params.add(key)

            for variant in variants:
                matches = all((param in given_params for param in variant))
                if matches:
                    break
            else:  # no break
                if len(variants) > 1:
                    variations = human_join(
                        ["(" + human_join([quote(arg) for arg in variant], final="and") + ")" for variant in variants]
                    )
                    msg = f"Missing required arguments; Expected either {variations} arguments to be given"
                else:
                    assert len(variants) > 0

                    # TODO: this error message is not deterministic
                    missing = list(set(variants[0]) - given_params)
                    if len(missing) > 1:
                        msg = f"Missing required arguments: {human_join([quote(arg) for arg in missing])}"
                    else:
                        msg = f"Missing required argument: {quote(missing[0])}"
                raise TypeError(msg)
            return func(*args, **kwargs)

        return wrapper  # type: ignore

    return inner


_K = TypeVar("_K")
_V = TypeVar("_V")


@overload
def strip_not_given(obj: None) -> None: ...


@overload
def strip_not_given(obj: Mapping[_K, _V | NotGiven]) -> dict[_K, _V]: ...


@overload
def strip_not_given(obj: object) -> object: ...


def strip_not_given(obj: object | None) -> object:
    """Remove all top-level keys where their values are instances of `NotGiven`"""
    if obj is None:
        return None

    if not is_mapping(obj):
        return obj

    return {key: value for key, value in obj.items() if not isinstance(value, NotGiven)}


def coerce_integer(val: str) -> int:
    return int(val, base=10)


def coerce_float(val: str) -> float:
    return float(val)


def coerce_boolean(val: str) -> bool:
    return val == "true" or val == "1" or val == "on"


def maybe_coerce_integer(val: str | None) -> int | None:
    if val is None:
        return None
    return coerce_integer(val)


def maybe_coerce_float(val: str | None) -> float | None:
    if val is None:
        return None
    return coerce_float(val)


def maybe_coerce_boolean(val: str | None) -> bool | None:
    if val is None:
        return None
    return coerce_boolean(val)


def removeprefix(string: str, prefix: str) -> str:
    """Remove a prefix from a string.

    Backport of `str.removeprefix` for Python < 3.9
    """
    if string.startswith(prefix):
        return string[len(prefix) :]
    return string


def removesuffix(string: str, suffix: str) -> str:
    """Remove a suffix from a string.

    Backport of `str.removesuffix` for Python < 3.9
    """
    if string.endswith(suffix):
        return string[: -len(suffix)]
    return string


def file_from_path(path: str) -> FileTypes:
    contents = Path(path).read_bytes()
    file_name = os.path.basename(path)
    return (file_name, contents)


def get_required_header(headers: HeadersLike, header: str) -> str:
    lower_header = header.lower()
    if is_mapping_t(headers):
        # mypy doesn't understand the type narrowing here
        for k, v in headers.items():  # type: ignore
            if k.lower() == lower_header and isinstance(v, str):
                return v

    # to deal with the case where the header looks like Stainless-Event-Id
    intercaps_header = re.sub(r"([^\w])(\w)", lambda pat: pat.group(1) + pat.group(2).upper(), header.capitalize())

    for normalized_header in [header, lower_header, header.upper(), intercaps_header]:
        value = headers.get(normalized_header)
        if value:
            return value

    raise ValueError(f"Could not find {header} header")


def get_async_library() -> str:
    try:
        return sniffio.current_async_library()
    except Exception:
        return "false"


def lru_cache(*, maxsize: int | None = 128) -> Callable[[CallableT], CallableT]:
    """A version of functools.lru_cache that retains the type signature
    for the wrapped function arguments.
    """
    wrapper = functools.lru_cache(  # noqa: TID251
        maxsize=maxsize,
    )
    return cast(Any, wrapper)  # type: ignore[no-any-return]


def json_safe(data: object) -> object:
    """Translates a mapping / sequence recursively in the same fashion
    as `pydantic` v2's `model_dump(mode="json")`.
    """
    if is_mapping(data):
        return {json_safe(key): json_safe(value) for key, value in data.items()}

    if is_iterable(data) and not isinstance(data, (str, bytes, bytearray)):
        return [json_safe(item) for item in data]

    if isinstance(data, (datetime, date)):
        return data.isoformat()

    return data



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from ..._models import BaseModel
from .device_info import DeviceInfo

__all__ = ["GetDeviceListResponse"]


class GetDeviceListResponse(BaseModel):
    data: List[DeviceInfo]
    """List of devices"""

    message: str
    """Response message"""

    total: float
    """Total number of devices"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["BoxDisplayResponse", "Resolution"]


class Resolution(BaseModel):
    height: float
    """Height of the screen"""

    width: float
    """Width of the screen"""


class BoxDisplayResponse(BaseModel):
    orientation: Literal["portrait", "landscapeLeft", "portraitUpsideDown", "landscapeRight"]
    """Orientation of the box"""

    resolution: Resolution
    """Resolution configuration"""



from typing import Dict, List, Union, Literal, Callable, Optional
from typing_extensions import Self

from gbox_sdk._types import NOT_GIVEN, NotGiven
from gbox_sdk._client import GboxClient
from gbox_sdk.wrapper.box.media import MediaOperator
from gbox_sdk.wrapper.box.proxy import ProxyOperator
from gbox_sdk.types.v1.linux_box import LinuxBox
from gbox_sdk.wrapper.box.action import ActionOperator
from gbox_sdk.wrapper.box.browser import BrowserOperator
from gbox_sdk.wrapper.box.storage import StorageOperator
from gbox_sdk.types.v1.android_box import AndroidBox
from gbox_sdk.wrapper.box.file_system import FileSystemOperator
from gbox_sdk.wrapper.box.websocket_client import WebSocketClient, WebSocketResult
from gbox_sdk.types.v1.box_display_response import BoxDisplayResponse
from gbox_sdk.types.v1.box_run_code_response import BoxRunCodeResponse
from gbox_sdk.types.v1.box_live_view_url_response import BoxLiveViewURLResponse
from gbox_sdk.types.v1.box_execute_commands_response import BoxExecuteCommandsResponse
from gbox_sdk.types.v1.box_web_terminal_url_response import BoxWebTerminalURLResponse


class BaseBox:
    """
    Base class for box operations, providing common interfaces for box lifecycle and actions.

    Attributes:
        client (GboxClient): The Gbox client instance used for API calls.
        data (Union[LinuxBox, AndroidBox]): The box data object.
        action (ActionOperator): Operator for box actions.
        fs (FileSystemOperator): Operator for file system actions.
        browser (BrowserOperator): Operator for browser actions.
    """

    def __init__(self, client: GboxClient, data: Union[LinuxBox, AndroidBox]):
        """
        Initialize a BaseBox instance.

        Args:
            client (GboxClient): The Gbox client instance.
            data (Union[LinuxBox, AndroidBox]): The box data object.
        """
        self.client = client
        self.data = data

        self.action = ActionOperator(self.client, self.data.id)
        self.fs = FileSystemOperator(self.client, self.data.id)
        self.browser = BrowserOperator(self.client, self.data.id)
        self.storage = StorageOperator(self.client, self.data.id)
        self.media = MediaOperator(self.client, self.data.id)
        self.proxy = ProxyOperator(self.client, self.data.id)

    def _sync_data(self) -> None:
        """
        Synchronize the box data with the latest state from the server.
        """
        res = self.client.v1.boxes.retrieve(box_id=self.data.id)
        self.data = res

    def start(self, *, wait: Union[bool, NotGiven] = NOT_GIVEN) -> Self:
        """
        Start the box.

        Args:
            wait: Whether to wait for the box to start.

        Returns:
            Self: The updated box instance for method chaining.

        Example:
            >>> box.start()
            >>> box.start(wait=True)
        """
        self.client.v1.boxes.start(box_id=self.data.id, wait=wait)
        self._sync_data()
        return self

    def stop(self, *, wait: Union[bool, NotGiven] = NOT_GIVEN) -> Self:
        """
        Stop the box.

        Args:
            wait: Whether to wait for the box to stop.

        Returns:
            Self: The updated box instance for method chaining.

        Example:
            >>> box.stop()
            >>> box.stop(wait=True)
        """
        self.client.v1.boxes.stop(box_id=self.data.id, wait=wait)
        self._sync_data()
        return self

    def terminate(self, *, wait: Union[bool, NotGiven] = NOT_GIVEN) -> Self:
        """
        Terminate the box.

        Args:
            wait: Whether to wait for the box to terminate.

        Returns:
            Self: The updated box instance for method chaining.

        Example:
            >>> box.terminate()
            >>> box.terminate(wait=True)
        """
        self.client.v1.boxes.terminate(box_id=self.data.id, wait=wait)
        self._sync_data()
        return self

    def display(self) -> BoxDisplayResponse:
        """
        Retrieve the current display properties for a running box.

        This endpoint
        provides details about the box's screen resolution, orientation, and other
        visual properties

        Returns:
            BoxDisplayResponse: The response containing the display properties.

        Example:
            >>> box.display()
        """
        return self.client.v1.boxes.display(box_id=self.data.id)

    def command(
        self,
        command: str,
        on_stdout: Optional[Callable[[str], None]] = None,
        on_stderr: Optional[Callable[[str], None]] = None,
        envs: Union[Dict[str, str], NotGiven] = NOT_GIVEN,
        api_timeout: Union[str, NotGiven] = NOT_GIVEN,
        working_dir: Union[str, NotGiven] = NOT_GIVEN,
    ) -> Union["BoxExecuteCommandsResponse", "WebSocketResult"]:
        """
        Execute shell commands in the box.

        Args:
            command: The command to run. Can be a single string or an array of strings

            envs: The environment variables to run the command

            api_timeout: The timeout of the command. If the command times out, the exit code will be 124.
                For example: 'timeout 5s sleep 10s' will result in exit code 124.

                Supported time units: ms (milliseconds), s (seconds), m (minutes), h (hours)
                Example formats: "500ms", "30s", "5m", "1h" Default: 30s

            working_dir: The working directory of the command. It not provided, the command will be run
                in the `box.config.workingDir` directory.

            on_stdout: Callback for stdout.

            on_stderr: Callback for stderr.

        Returns:
            Union[BoxExecuteCommandsResponse, WebSocketResult]: The response containing the command execution result.

        Example:
            >>> box.command(commands="ls -l", on_stdout=lambda x: print(x), on_stderr=lambda x: print(x))
        """
        if on_stdout is not None or on_stderr is not None:
            return self._command_via_websocket(command, on_stdout, on_stderr, envs, api_timeout, working_dir)

        return self.client.v1.boxes.execute_commands(
            box_id=self.data.id,
            command=command,
            envs=envs,
            api_timeout=api_timeout,
            working_dir=working_dir,
        )

    def _command_via_websocket(
        self,
        commands: Union[List[str], str],
        on_stdout: Optional[Callable[[str], None]] = None,
        on_stderr: Optional[Callable[[str], None]] = None,
        envs: Union[Dict[str, str], NotGiven] = NOT_GIVEN,
        api_timeout: Union[str, NotGiven] = NOT_GIVEN,
        working_dir: Union[str, NotGiven] = NOT_GIVEN,
    ) -> "WebSocketResult":
        """
        Execute commands via WebSocket with streaming output.

        This method runs the WebSocket execution in a new event loop if one is not already running.
        """
        try:
            websocket_response = self.client.v1.boxes.websocket_url(box_id=self.data.id)
            websocket_url = websocket_response.command

            websocket_client = WebSocketClient(websocket_url, self.client.api_key)

            import asyncio

            try:
                loop = asyncio.get_running_loop()
                return asyncio.run_coroutine_threadsafe(
                    websocket_client.execute_command(
                        commands=commands,
                        on_stdout=on_stdout,
                        on_stderr=on_stderr,
                        envs=envs,
                        api_timeout=api_timeout,
                        working_dir=working_dir,
                    ),
                    loop,
                ).result()
            except RuntimeError:
                return asyncio.run(
                    websocket_client.execute_command(
                        commands=commands,
                        on_stdout=on_stdout,
                        on_stderr=on_stderr,
                        envs=envs,
                        api_timeout=api_timeout,
                        working_dir=working_dir,
                    )
                )

        except Exception as e:
            raise RuntimeError(f"Failed to execute command via WebSocket: {e}") from e

    def run_code(
        self,
        code: str,
        argv: Union[List[str], NotGiven] = NOT_GIVEN,
        envs: Union[Dict[str, str], NotGiven] = NOT_GIVEN,
        language: Union[Literal["bash", "python", "typescript"], NotGiven] = NOT_GIVEN,
        api_timeout: Union[str, NotGiven] = NOT_GIVEN,
        working_dir: Union[str, NotGiven] = NOT_GIVEN,
        on_stdout: Optional[Callable[[str], None]] = None,
        on_stderr: Optional[Callable[[str], None]] = None,
    ) -> Union["BoxRunCodeResponse", "WebSocketResult"]:
        """
        Run code in the box.

        Args:
            code: The code to run

            argv: The arguments to run the code. For example, if you want to run "python index.py
                --help", you should pass ["--help"] as arguments.

            envs: The environment variables to run the code

            language: The language of the code.

            api_timeout: The timeout of the code execution. If the code execution times out, the exit
                code will be 124.

                Supported time units: ms (milliseconds), s (seconds), m (minutes), h (hours)
                Example formats: "500ms", "30s", "5m", "1h" Default: 30s

            working_dir: The working directory of the code. It not provided, the code will be run in the
                `box.config.workingDir` directory.

            on_stdout: Callback for stdout.

            on_stderr: Callback for stderr.
        Returns:
            Union[BoxRunCodeResponse, WebSocketResult]: The response containing the code execution result.

        Example:
            >>> box.run_code(
            ...     code="print('Hello, World!')",
            ...     language="python",
            ...     on_stdout=lambda x: print(x),
            ...     on_stderr=lambda x: print(x),
            ... )
        """

        if on_stdout is not None or on_stderr is not None:
            return self._run_code_via_websocket(
                code=code,
                argv=argv,
                envs=envs,
                language=language,
                api_timeout=api_timeout,
                working_dir=working_dir,
                on_stdout=on_stdout,
                on_stderr=on_stderr,
            )

        return self.client.v1.boxes.run_code(
            box_id=self.data.id,
            code=code,
            argv=argv,
            envs=envs,
            language=language,
            api_timeout=api_timeout,
            working_dir=working_dir,
        )

    def _run_code_via_websocket(
        self,
        code: str,
        argv: Union[List[str], NotGiven] = NOT_GIVEN,
        envs: Union[Dict[str, str], NotGiven] = NOT_GIVEN,
        language: Union[Literal["bash", "python", "typescript"], NotGiven] = NOT_GIVEN,
        api_timeout: Union[str, NotGiven] = NOT_GIVEN,
        working_dir: Union[str, NotGiven] = NOT_GIVEN,
        on_stdout: Optional[Callable[[str], None]] = None,
        on_stderr: Optional[Callable[[str], None]] = None,
    ) -> "WebSocketResult":
        """
        Run code via WebSocket with streaming output.

        This method runs the WebSocket execution in a new event loop if one is not already running.
        """
        try:
            websocket_response = self.client.v1.boxes.websocket_url(box_id=self.data.id)
            websocket_url = websocket_response.run_code

            websocket_client = WebSocketClient(websocket_url, self.client.api_key)

            import asyncio

            try:
                loop = asyncio.get_running_loop()
                return asyncio.run_coroutine_threadsafe(
                    websocket_client.run_code(
                        code=code,
                        on_stdout=on_stdout,
                        on_stderr=on_stderr,
                        argv=argv,
                        envs=envs,
                        language=language,
                        api_timeout=api_timeout,
                        working_dir=working_dir,
                    ),
                    loop,
                ).result()
            except RuntimeError:
                return asyncio.run(
                    websocket_client.run_code(
                        code=code,
                        on_stdout=on_stdout,
                        on_stderr=on_stderr,
                        argv=argv,
                        envs=envs,
                        language=language,
                        api_timeout=api_timeout,
                        working_dir=working_dir,
                    )
                )

        except Exception as e:
            raise RuntimeError(f"Failed to run code via WebSocket: {e}") from e

    def live_view(
        self,
        *,
        expires_in: Union[str, NotGiven] = NOT_GIVEN,
    ) -> BoxLiveViewURLResponse:
        """
        Get the live view URL for the box.

        Args:
            expires_in: The duration for the live view URL to be valid.
        Returns:
            BoxLiveViewURLResponse: The response containing the live view URL.
        """
        return self.client.v1.boxes.live_view_url(
            box_id=self.data.id,
            expires_in=expires_in,
        )

    def web_terminal(
        self,
        *,
        expires_in: Union[str, NotGiven] = NOT_GIVEN,
    ) -> BoxWebTerminalURLResponse:
        """
        Get the web terminal URL for the box.

        Args:
            body (BoxWebTerminalURLParams): Parameters for web terminal URL.
        Returns:
            BoxWebTerminalURLResponse: The response containing the web terminal URL.
        """
        return self.client.v1.boxes.web_terminal_url(
            box_id=self.data.id,
            expires_in=expires_in,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["ActionRewindExtractParams"]


class ActionRewindExtractParams(TypedDict, total=False):
    duration: str
    """How far back in time to rewind for extracting recorded video.

    This specifies the duration to go back from the current moment (e.g., '30s'
    rewinds 30 seconds to get recent recorded activity). Default is 30s, max is 5m.

    Supported time units: ms (milliseconds), s (seconds), m (minutes), h (hours)
    Example formats: "500ms", "30s", "5m", "1h" Maximum allowed: 5m
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from ...._models import BaseModel

__all__ = ["MediaGetMediaSupportResponse"]


class MediaGetMediaSupportResponse(BaseModel):
    photo: List[str]
    """Supported photo extensions"""

    video: List[str]
    """Supported video extensions"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["BoxLiveViewURLResponse"]


class BoxLiveViewURLResponse(BaseModel):
    raw_url: str = FieldInfo(alias="rawUrl")
    """
    Raw live view url without additional layout content, typically used for
    embedding into your own application
    """

    url: str
    """
    Live view url with Gbox interface and basic information, typically used for
    real-time observation of box usage status
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, Annotated, TypedDict

from ...._types import SequenceNotStr
from ...._utils import PropertyInfo
from .action_common_options_param import ActionCommonOptionsParam

__all__ = ["ActionAIParams", "Settings"]


class ActionAIParams(TypedDict, total=False):
    instruction: Required[str]
    """
    Direct instruction of the UI action to perform (e.g., 'click the login button',
    'input username in the email field', 'scroll down', 'swipe left')
    """

    background: str
    """The background of the UI action to perform.

    The purpose of background is to let the action executor to understand the
    context of why the instruction is given including important previous actions and
    observations
    """

    include_screenshot: Annotated[bool, PropertyInfo(alias="includeScreenshot")]
    """⚠️ DEPRECATED: Use `options.screenshot.phases` instead.

    This field will be ignored when `options.screenshot` is provided. Whether to
    include screenshots in the action response. If false, the screenshot object will
    still be returned but with empty URIs. Default is false.
    """

    options: ActionCommonOptionsParam
    """Action common options"""

    output_format: Annotated[Literal["base64", "storageKey"], PropertyInfo(alias="outputFormat")]
    """⚠️ DEPRECATED: Use `options.screenshot.outputFormat` instead.

    Type of the URI. default is base64. This field will be ignored when
    `options.screenshot` is provided.
    """

    presigned_expires_in: Annotated[str, PropertyInfo(alias="presignedExpiresIn")]
    """⚠️ DEPRECATED: Use `options.screenshot.presignedExpiresIn` instead.

    Presigned url expires in. Only takes effect when outputFormat is storageKey.
    This field will be ignored when `options.screenshot` is provided.

    Supported time units: ms (milliseconds), s (seconds), m (minutes), h (hours)
    Example formats: "500ms", "30s", "5m", "1h" Default: 30m
    """

    screenshot_delay: Annotated[str, PropertyInfo(alias="screenshotDelay")]
    """⚠️ DEPRECATED: Use `options.screenshot.delay` instead.

    This field will be ignored when `options.screenshot` is provided.

    Delay after performing the action, before taking the final screenshot.

    Execution flow:

    1. Take screenshot before action
    2. Perform the action
    3. Wait for screenshotDelay (this parameter)
    4. Take screenshot after action

    Example: '500ms' means wait 500ms after the action before capturing the final
    screenshot.

    Supported time units: ms (milliseconds), s (seconds), m (minutes), h (hours)
    Example formats: "500ms", "30s", "5m", "1h" Default: 500ms Maximum allowed: 30s
    """

    settings: Settings
    """AI action settings"""

    stream: bool
    """Whether to stream progress events using Server-Sent Events (SSE).

    When true, the API returns an event stream. When false or omitted, the API
    returns a normal JSON response.
    """


class Settings(TypedDict, total=False):
    disable_actions: Annotated[SequenceNotStr[str], PropertyInfo(alias="disableActions")]
    """Whether disable actions"""

    system_prompt: Annotated[str, PropertyInfo(alias="systemPrompt")]
    """
    System prompt that defines the AI's behavior and capabilities when executing UI
    actions. This prompt instructs the AI on how to interpret the screen, understand
    user instructions, and determine the appropriate UI actions to take. A
    well-crafted system prompt can significantly improve the accuracy and
    reliability of AI-driven UI automation. If not provided, uses the default
    computer use instruction template that includes basic screen interaction
    guidelines.
    """



from typing_extensions import Union

from gbox_sdk._types import NOT_GIVEN, NotGiven
from gbox_sdk._client import GboxClient


class StorageOperator:
    """
    Operator class for managing storage via the Gbox SDK.
    Provides an interface to interact with and control storage.
    """

    def __init__(self, client: GboxClient, box_id: str):
        """
        Initialize a StorageOperator instance.

        Args:
            client (GboxClient): The Gbox client used for API communication.
            box_id (str): The box id.
        """
        self.client = client
        self.box_id = box_id

    def create_presigned_url(self, storage_key: str, *, expires_in: Union[str, NotGiven] = NOT_GIVEN) -> str:
        """
        Create a presigned url for a storage key.

        This endpoint provides a presigned url
        for a storage key, which can be used to download the file from the storage.

        Args:
            storage_key: Storage key

            expires_in: Presigned url expires in
                Supported time units: ms (milliseconds), s (seconds), m (minutes), h (hours)
                Example formats: "500ms", "30s", "5m", "1h" Default: 30m Maximum allowed: 6h
        """
        return self.client.v1.boxes.storage.presigned_url(
            box_id=self.box_id, storage_key=storage_key, expires_in=expires_in
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Union
from typing_extensions import TypeAlias

from .media_photo import MediaPhoto
from .media_video import MediaVideo

__all__ = ["MediaGetMediaResponse"]

MediaGetMediaResponse: TypeAlias = Union[MediaPhoto, MediaVideo]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from typing_extensions import Literal, Required, Annotated, TypeAlias, TypedDict

from ...._utils import PropertyInfo
from .action_common_options_param import ActionCommonOptionsParam

__all__ = ["ActionClickParams", "Click", "ClickByNaturalLanguage"]


class Click(TypedDict, total=False):
    x: Required[float]
    """X coordinate of the click"""

    y: Required[float]
    """Y coordinate of the click"""

    button: Literal["left", "right", "middle"]
    """Mouse button to click"""

    double: bool
    """Whether to perform a double click"""

    include_screenshot: Annotated[bool, PropertyInfo(alias="includeScreenshot")]
    """⚠️ DEPRECATED: Use `options.screenshot.phases` instead.

    This field will be ignored when `options.screenshot` is provided. Whether to
    include screenshots in the action response. If false, the screenshot object will
    still be returned but with empty URIs. Default is false.
    """

    options: ActionCommonOptionsParam
    """Action common options"""

    output_format: Annotated[Literal["base64", "storageKey"], PropertyInfo(alias="outputFormat")]
    """⚠️ DEPRECATED: Use `options.screenshot.outputFormat` instead.

    Type of the URI. default is base64. This field will be ignored when
    `options.screenshot` is provided.
    """

    presigned_expires_in: Annotated[str, PropertyInfo(alias="presignedExpiresIn")]
    """⚠️ DEPRECATED: Use `options.screenshot.presignedExpiresIn` instead.

    Presigned url expires in. Only takes effect when outputFormat is storageKey.
    This field will be ignored when `options.screenshot` is provided.

    Supported time units: ms (milliseconds), s (seconds), m (minutes), h (hours)
    Example formats: "500ms", "30s", "5m", "1h" Default: 30m
    """

    screenshot_delay: Annotated[str, PropertyInfo(alias="screenshotDelay")]
    """⚠️ DEPRECATED: Use `options.screenshot.delay` instead.

    This field will be ignored when `options.screenshot` is provided.

    Delay after performing the action, before taking the final screenshot.

    Execution flow:

    1. Take screenshot before action
    2. Perform the action
    3. Wait for screenshotDelay (this parameter)
    4. Take screenshot after action

    Example: '500ms' means wait 500ms after the action before capturing the final
    screenshot.

    Supported time units: ms (milliseconds), s (seconds), m (minutes), h (hours)
    Example formats: "500ms", "30s", "5m", "1h" Default: 500ms Maximum allowed: 30s
    """


class ClickByNaturalLanguage(TypedDict, total=False):
    target: Required[str]
    """
    Describe the target to operate using natural language, e.g., 'login button' or
    'Chrome'.
    """

    button: Literal["left", "right", "middle"]
    """Mouse button to click"""

    double: bool
    """Whether to perform a double click"""

    include_screenshot: Annotated[bool, PropertyInfo(alias="includeScreenshot")]
    """⚠️ DEPRECATED: Use `options.screenshot.phases` instead.

    This field will be ignored when `options.screenshot` is provided. Whether to
    include screenshots in the action response. If false, the screenshot object will
    still be returned but with empty URIs. Default is false.
    """

    options: ActionCommonOptionsParam
    """Action common options"""

    output_format: Annotated[Literal["base64", "storageKey"], PropertyInfo(alias="outputFormat")]
    """⚠️ DEPRECATED: Use `options.screenshot.outputFormat` instead.

    Type of the URI. default is base64. This field will be ignored when
    `options.screenshot` is provided.
    """

    presigned_expires_in: Annotated[str, PropertyInfo(alias="presignedExpiresIn")]
    """⚠️ DEPRECATED: Use `options.screenshot.presignedExpiresIn` instead.

    Presigned url expires in. Only takes effect when outputFormat is storageKey.
    This field will be ignored when `options.screenshot` is provided.

    Supported time units: ms (milliseconds), s (seconds), m (minutes), h (hours)
    Example formats: "500ms", "30s", "5m", "1h" Default: 30m
    """

    screenshot_delay: Annotated[str, PropertyInfo(alias="screenshotDelay")]
    """⚠️ DEPRECATED: Use `options.screenshot.delay` instead.

    This field will be ignored when `options.screenshot` is provided.

    Delay after performing the action, before taking the final screenshot.

    Execution flow:

    1. Take screenshot before action
    2. Perform the action
    3. Wait for screenshotDelay (this parameter)
    4. Take screenshot after action

    Example: '500ms' means wait 500ms after the action before capturing the final
    screenshot.

    Supported time units: ms (milliseconds), s (seconds), m (minutes), h (hours)
    Example formats: "500ms", "30s", "5m", "1h" Default: 500ms Maximum allowed: 30s
    """


ActionClickParams: TypeAlias = Union[Click, ClickByNaturalLanguage]



from typing import Union

from gbox_sdk.types.v1.linux_box import LinuxBox
from gbox_sdk.types.v1.android_box import AndroidBox


def is_android_box(box: Union[AndroidBox, LinuxBox]) -> bool:
    return getattr(box, "type", None) == "android"


def is_linux_box(box: Union[AndroidBox, LinuxBox]) -> bool:
    return getattr(box, "type", None) == "linux"



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["DeviceInfo"]


class DeviceInfo(BaseModel):
    device_id: str = FieldInfo(alias="deviceId")
    """Device ID"""

    enable: str
    """Device enable status"""

    is_idle: bool = FieldInfo(alias="isIdle")
    """Whether device is idle"""

    product_model: str = FieldInfo(alias="productModel")
    """Product model from ro.product.model"""

    provider_id: str = FieldInfo(alias="providerId")
    """Provider ID"""

    provider_type: str = FieldInfo(alias="providerType")
    """Provider type"""

    status: str
    """Device status"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ...._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ...._utils import maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._base_client import make_request_options
from ....types.v1.boxes import storage_presigned_url_params

__all__ = ["StorageResource", "AsyncStorageResource"]


class StorageResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> StorageResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/babelcloud/gbox-sdk-py#accessing-raw-response-data-eg-headers
        """
        return StorageResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> StorageResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/babelcloud/gbox-sdk-py#with_streaming_response
        """
        return StorageResourceWithStreamingResponse(self)

    def presigned_url(
        self,
        box_id: str,
        *,
        storage_key: str,
        expires_in: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> str:
        """Create a presigned url for a storage key.

        This endpoint provides a presigned url
        for a storage key, which can be used to download the file from the storage.

        Args:
          storage_key: Storage key

          expires_in: Presigned url expires in

              Supported time units: ms (milliseconds), s (seconds), m (minutes), h (hours)
              Example formats: "500ms", "30s", "5m", "1h" Default: 30m Maximum allowed: 6h

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        return self._post(
            f"/boxes/{box_id}/storage/presigned-url",
            body=maybe_transform(
                {
                    "storage_key": storage_key,
                    "expires_in": expires_in,
                },
                storage_presigned_url_params.StoragePresignedURLParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=str,
        )


class AsyncStorageResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncStorageResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/babelcloud/gbox-sdk-py#accessing-raw-response-data-eg-headers
        """
        return AsyncStorageResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncStorageResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/babelcloud/gbox-sdk-py#with_streaming_response
        """
        return AsyncStorageResourceWithStreamingResponse(self)

    async def presigned_url(
        self,
        box_id: str,
        *,
        storage_key: str,
        expires_in: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> str:
        """Create a presigned url for a storage key.

        This endpoint provides a presigned url
        for a storage key, which can be used to download the file from the storage.

        Args:
          storage_key: Storage key

          expires_in: Presigned url expires in

              Supported time units: ms (milliseconds), s (seconds), m (minutes), h (hours)
              Example formats: "500ms", "30s", "5m", "1h" Default: 30m Maximum allowed: 6h

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        return await self._post(
            f"/boxes/{box_id}/storage/presigned-url",
            body=await async_maybe_transform(
                {
                    "storage_key": storage_key,
                    "expires_in": expires_in,
                },
                storage_presigned_url_params.StoragePresignedURLParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=str,
        )


class StorageResourceWithRawResponse:
    def __init__(self, storage: StorageResource) -> None:
        self._storage = storage

        self.presigned_url = to_raw_response_wrapper(
            storage.presigned_url,
        )


class AsyncStorageResourceWithRawResponse:
    def __init__(self, storage: AsyncStorageResource) -> None:
        self._storage = storage

        self.presigned_url = async_to_raw_response_wrapper(
            storage.presigned_url,
        )


class StorageResourceWithStreamingResponse:
    def __init__(self, storage: StorageResource) -> None:
        self._storage = storage

        self.presigned_url = to_streamed_response_wrapper(
            storage.presigned_url,
        )


class AsyncStorageResourceWithStreamingResponse:
    def __init__(self, storage: AsyncStorageResource) -> None:
        self._storage = storage

        self.presigned_url = async_to_streamed_response_wrapper(
            storage.presigned_url,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from datetime import datetime
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from ...._models import BaseModel

__all__ = ["MediaPhoto"]


class MediaPhoto(BaseModel):
    last_modified: datetime = FieldInfo(alias="lastModified")
    """Last modified time of the photo"""

    mime_type: str = FieldInfo(alias="mimeType")
    """MIME type of the photo"""

    name: str
    """Name of the photo"""

    path: str
    """Full path to the photo in the box"""

    size: str
    """Size of the photo"""

    type: Literal["photo"]
    """Photo type indicator"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from pydantic import Field as FieldInfo

from ...._models import BaseModel

__all__ = ["AndroidApp"]


class AndroidApp(BaseModel):
    activity_class_name: str = FieldInfo(alias="activityClassName")
    """Activity class name"""

    activity_name: str = FieldInfo(alias="activityName")
    """Activity name"""

    package_name: str = FieldInfo(alias="packageName")
    """App package name"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from ...._models import BaseModel

__all__ = ["ActionScreenshotOptions"]


class ActionScreenshotOptions(BaseModel):
    delay: Optional[str] = None
    """Delay after performing the action, before taking the final screenshot.

    Execution flow:

    1. Take screenshot before action
    2. Perform the action
    3. Wait for screenshotDelay (this parameter)
    4. Take screenshot after action

    Example: '500ms' means wait 500ms after the action before capturing the final
    screenshot.

    Supported time units: ms (milliseconds), s (seconds), m (minutes), h (hours)
    Example formats: "500ms", "30s", "5m", "1h" Default: 500ms Maximum allowed: 30s
    """

    output_format: Optional[Literal["base64", "storageKey"]] = FieldInfo(alias="outputFormat", default=None)
    """Type of the URI. default is base64."""

    phases: Optional[List[Literal["before", "after", "trace"]]] = None
    """Specify which screenshot phases to capture.

    Available options:

    - before: Screenshot before the action
    - after: Screenshot after the action
    - trace: Screenshot with operation trace

    Default captures all three phases. Can specify one or multiple in an array. If
    empty array is provided, no screenshots will be taken.
    """

    presigned_expires_in: Optional[str] = FieldInfo(alias="presignedExpiresIn", default=None)
    """Presigned url expires in. Only takes effect when outputFormat is storageKey.

    Supported time units: ms (milliseconds), s (seconds), m (minutes), h (hours)
    Example formats: "500ms", "30s", "5m", "1h" Default: 30m
    """



from gbox_sdk._client import GboxClient
from gbox_sdk.wrapper.box.base import BaseBox
from gbox_sdk.types.v1.android_box import AndroidBox
from gbox_sdk.wrapper.box.android.app_manager import AndroidAppManager
from gbox_sdk.wrapper.box.android.pkg_manager import AndroidPkgManager
from gbox_sdk.types.v1.boxes.android_get_connect_address_response import AndroidGetConnectAddressResponse


class AndroidBoxOperator(BaseBox):
    """
    Operator class for managing Android boxes, providing access to app and package management functionalities.

    Attributes:
        app (AndroidAppManager): Manager for Android app operations.
        pkg (AndroidPkgManager): Manager for Android package operations.
    """

    def __init__(self, client: GboxClient, data: AndroidBox):
        """
        Initialize an AndroidBoxOperator instance.

        Args:
            client (GboxClient): The API client used for communication.
            data (AndroidBox): The Android box data object.
        """
        super().__init__(client, data)
        self.app = AndroidAppManager(client, data)
        self.pkg = AndroidPkgManager(client, data)

    def get_connect_address(self) -> AndroidGetConnectAddressResponse:
        """
        Get the connect address for the Android box.

        Returns:
            AndroidGetConnectAddressResponse: The connect address for the Android box.
        """
        return self.client.v1.boxes.android.get_connect_address(box_id=self.data.id)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import httpx
import pytest
from respx import MockRouter

from gbox_sdk import GboxClient, AsyncGboxClient
from tests.utils import assert_matches_type
from gbox_sdk._response import (
    BinaryAPIResponse,
    AsyncBinaryAPIResponse,
    StreamedBinaryAPIResponse,
    AsyncStreamedBinaryAPIResponse,
)
from gbox_sdk.types.v1.boxes import (
    AndroidApp,
    AndroidPkg,
    AndroidInstallResponse,
    AndroidListAppResponse,
    AndroidListPkgResponse,
    AndroidListPkgSimpleResponse,
    AndroidListActivitiesResponse,
    AndroidGetConnectAddressResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestAndroid:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    def test_method_backup(self, client: GboxClient, respx_mock: MockRouter) -> None:
        respx_mock.post("/boxes/c9bdc193-b54b-4ddb-a035-5ac0c598d32d/android/packages/com.example.myapp/backup").mock(
            return_value=httpx.Response(200, json={"foo": "bar"})
        )
        android = client.v1.boxes.android.backup(
            package_name="com.example.myapp",
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        )
        assert android.is_closed
        assert android.json() == {"foo": "bar"}
        assert cast(Any, android.is_closed) is True
        assert isinstance(android, BinaryAPIResponse)

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    def test_raw_response_backup(self, client: GboxClient, respx_mock: MockRouter) -> None:
        respx_mock.post("/boxes/c9bdc193-b54b-4ddb-a035-5ac0c598d32d/android/packages/com.example.myapp/backup").mock(
            return_value=httpx.Response(200, json={"foo": "bar"})
        )

        android = client.v1.boxes.android.with_raw_response.backup(
            package_name="com.example.myapp",
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        )

        assert android.is_closed is True
        assert android.http_request.headers.get("X-Stainless-Lang") == "python"
        assert android.json() == {"foo": "bar"}
        assert isinstance(android, BinaryAPIResponse)

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    def test_streaming_response_backup(self, client: GboxClient, respx_mock: MockRouter) -> None:
        respx_mock.post("/boxes/c9bdc193-b54b-4ddb-a035-5ac0c598d32d/android/packages/com.example.myapp/backup").mock(
            return_value=httpx.Response(200, json={"foo": "bar"})
        )
        with client.v1.boxes.android.with_streaming_response.backup(
            package_name="com.example.myapp",
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        ) as android:
            assert not android.is_closed
            assert android.http_request.headers.get("X-Stainless-Lang") == "python"

            assert android.json() == {"foo": "bar"}
            assert cast(Any, android.is_closed) is True
            assert isinstance(android, StreamedBinaryAPIResponse)

        assert cast(Any, android.is_closed) is True

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    def test_path_params_backup(self, client: GboxClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `box_id` but received ''"):
            client.v1.boxes.android.with_raw_response.backup(
                package_name="com.example.myapp",
                box_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `package_name` but received ''"):
            client.v1.boxes.android.with_raw_response.backup(
                package_name="",
                box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
            )

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    def test_method_backup_all(self, client: GboxClient, respx_mock: MockRouter) -> None:
        respx_mock.post("/boxes/c9bdc193-b54b-4ddb-a035-5ac0c598d32d/android/packages/backup-all").mock(
            return_value=httpx.Response(200, json={"foo": "bar"})
        )
        android = client.v1.boxes.android.backup_all(
            "c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        )
        assert android.is_closed
        assert android.json() == {"foo": "bar"}
        assert cast(Any, android.is_closed) is True
        assert isinstance(android, BinaryAPIResponse)

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    def test_raw_response_backup_all(self, client: GboxClient, respx_mock: MockRouter) -> None:
        respx_mock.post("/boxes/c9bdc193-b54b-4ddb-a035-5ac0c598d32d/android/packages/backup-all").mock(
            return_value=httpx.Response(200, json={"foo": "bar"})
        )

        android = client.v1.boxes.android.with_raw_response.backup_all(
            "c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        )

        assert android.is_closed is True
        assert android.http_request.headers.get("X-Stainless-Lang") == "python"
        assert android.json() == {"foo": "bar"}
        assert isinstance(android, BinaryAPIResponse)

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    def test_streaming_response_backup_all(self, client: GboxClient, respx_mock: MockRouter) -> None:
        respx_mock.post("/boxes/c9bdc193-b54b-4ddb-a035-5ac0c598d32d/android/packages/backup-all").mock(
            return_value=httpx.Response(200, json={"foo": "bar"})
        )
        with client.v1.boxes.android.with_streaming_response.backup_all(
            "c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        ) as android:
            assert not android.is_closed
            assert android.http_request.headers.get("X-Stainless-Lang") == "python"

            assert android.json() == {"foo": "bar"}
            assert cast(Any, android.is_closed) is True
            assert isinstance(android, StreamedBinaryAPIResponse)

        assert cast(Any, android.is_closed) is True

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    def test_path_params_backup_all(self, client: GboxClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `box_id` but received ''"):
            client.v1.boxes.android.with_raw_response.backup_all(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_close(self, client: GboxClient) -> None:
        android = client.v1.boxes.android.close(
            package_name="com.example.myapp",
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        )
        assert android is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_close(self, client: GboxClient) -> None:
        response = client.v1.boxes.android.with_raw_response.close(
            package_name="com.example.myapp",
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        android = response.parse()
        assert android is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_close(self, client: GboxClient) -> None:
        with client.v1.boxes.android.with_streaming_response.close(
            package_name="com.example.myapp",
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            android = response.parse()
            assert android is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_close(self, client: GboxClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `box_id` but received ''"):
            client.v1.boxes.android.with_raw_response.close(
                package_name="com.example.myapp",
                box_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `package_name` but received ''"):
            client.v1.boxes.android.with_raw_response.close(
                package_name="",
                box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_close_all(self, client: GboxClient) -> None:
        android = client.v1.boxes.android.close_all(
            "c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        )
        assert android is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_close_all(self, client: GboxClient) -> None:
        response = client.v1.boxes.android.with_raw_response.close_all(
            "c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        android = response.parse()
        assert android is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_close_all(self, client: GboxClient) -> None:
        with client.v1.boxes.android.with_streaming_response.close_all(
            "c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            android = response.parse()
            assert android is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_close_all(self, client: GboxClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `box_id` but received ''"):
            client.v1.boxes.android.with_raw_response.close_all(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_get(self, client: GboxClient) -> None:
        android = client.v1.boxes.android.get(
            package_name="com.example.myapp",
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        )
        assert_matches_type(AndroidPkg, android, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_get(self, client: GboxClient) -> None:
        response = client.v1.boxes.android.with_raw_response.get(
            package_name="com.example.myapp",
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        android = response.parse()
        assert_matches_type(AndroidPkg, android, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_get(self, client: GboxClient) -> None:
        with client.v1.boxes.android.with_streaming_response.get(
            package_name="com.example.myapp",
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            android = response.parse()
            assert_matches_type(AndroidPkg, android, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_get(self, client: GboxClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `box_id` but received ''"):
            client.v1.boxes.android.with_raw_response.get(
                package_name="com.example.myapp",
                box_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `package_name` but received ''"):
            client.v1.boxes.android.with_raw_response.get(
                package_name="",
                box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_get_app(self, client: GboxClient) -> None:
        android = client.v1.boxes.android.get_app(
            package_name="com.example.myapp",
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        )
        assert_matches_type(AndroidApp, android, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_get_app(self, client: GboxClient) -> None:
        response = client.v1.boxes.android.with_raw_response.get_app(
            package_name="com.example.myapp",
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        android = response.parse()
        assert_matches_type(AndroidApp, android, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_get_app(self, client: GboxClient) -> None:
        with client.v1.boxes.android.with_streaming_response.get_app(
            package_name="com.example.myapp",
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            android = response.parse()
            assert_matches_type(AndroidApp, android, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_get_app(self, client: GboxClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `box_id` but received ''"):
            client.v1.boxes.android.with_raw_response.get_app(
                package_name="com.example.myapp",
                box_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `package_name` but received ''"):
            client.v1.boxes.android.with_raw_response.get_app(
                package_name="",
                box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_get_connect_address(self, client: GboxClient) -> None:
        android = client.v1.boxes.android.get_connect_address(
            "c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        )
        assert_matches_type(AndroidGetConnectAddressResponse, android, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_get_connect_address(self, client: GboxClient) -> None:
        response = client.v1.boxes.android.with_raw_response.get_connect_address(
            "c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        android = response.parse()
        assert_matches_type(AndroidGetConnectAddressResponse, android, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_get_connect_address(self, client: GboxClient) -> None:
        with client.v1.boxes.android.with_streaming_response.get_connect_address(
            "c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            android = response.parse()
            assert_matches_type(AndroidGetConnectAddressResponse, android, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_get_connect_address(self, client: GboxClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `box_id` but received ''"):
            client.v1.boxes.android.with_raw_response.get_connect_address(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_install_overload_1(self, client: GboxClient) -> None:
        android = client.v1.boxes.android.install(
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
            apk=b"raw file contents",
        )
        assert_matches_type(AndroidInstallResponse, android, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_install_with_all_params_overload_1(self, client: GboxClient) -> None:
        android = client.v1.boxes.android.install(
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
            apk=b"raw file contents",
            open=False,
        )
        assert_matches_type(AndroidInstallResponse, android, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_install_overload_1(self, client: GboxClient) -> None:
        response = client.v1.boxes.android.with_raw_response.install(
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
            apk=b"raw file contents",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        android = response.parse()
        assert_matches_type(AndroidInstallResponse, android, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_install_overload_1(self, client: GboxClient) -> None:
        with client.v1.boxes.android.with_streaming_response.install(
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
            apk=b"raw file contents",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            android = response.parse()
            assert_matches_type(AndroidInstallResponse, android, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_install_overload_1(self, client: GboxClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `box_id` but received ''"):
            client.v1.boxes.android.with_raw_response.install(
                box_id="",
                apk=b"raw file contents",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_install_overload_2(self, client: GboxClient) -> None:
        android = client.v1.boxes.android.install(
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
            apk="https://example.com/app.apk",
        )
        assert_matches_type(AndroidInstallResponse, android, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_install_with_all_params_overload_2(self, client: GboxClient) -> None:
        android = client.v1.boxes.android.install(
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
            apk="https://example.com/app.apk",
            open=False,
        )
        assert_matches_type(AndroidInstallResponse, android, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_install_overload_2(self, client: GboxClient) -> None:
        response = client.v1.boxes.android.with_raw_response.install(
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
            apk="https://example.com/app.apk",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        android = response.parse()
        assert_matches_type(AndroidInstallResponse, android, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_install_overload_2(self, client: GboxClient) -> None:
        with client.v1.boxes.android.with_streaming_response.install(
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
            apk="https://example.com/app.apk",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            android = response.parse()
            assert_matches_type(AndroidInstallResponse, android, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_install_overload_2(self, client: GboxClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `box_id` but received ''"):
            client.v1.boxes.android.with_raw_response.install(
                box_id="",
                apk="https://example.com/app.apk",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_activities(self, client: GboxClient) -> None:
        android = client.v1.boxes.android.list_activities(
            package_name="com.example.myapp",
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        )
        assert_matches_type(AndroidListActivitiesResponse, android, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list_activities(self, client: GboxClient) -> None:
        response = client.v1.boxes.android.with_raw_response.list_activities(
            package_name="com.example.myapp",
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        android = response.parse()
        assert_matches_type(AndroidListActivitiesResponse, android, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list_activities(self, client: GboxClient) -> None:
        with client.v1.boxes.android.with_streaming_response.list_activities(
            package_name="com.example.myapp",
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            android = response.parse()
            assert_matches_type(AndroidListActivitiesResponse, android, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_list_activities(self, client: GboxClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `box_id` but received ''"):
            client.v1.boxes.android.with_raw_response.list_activities(
                package_name="com.example.myapp",
                box_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `package_name` but received ''"):
            client.v1.boxes.android.with_raw_response.list_activities(
                package_name="",
                box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_app(self, client: GboxClient) -> None:
        android = client.v1.boxes.android.list_app(
            "c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        )
        assert_matches_type(AndroidListAppResponse, android, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list_app(self, client: GboxClient) -> None:
        response = client.v1.boxes.android.with_raw_response.list_app(
            "c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        android = response.parse()
        assert_matches_type(AndroidListAppResponse, android, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list_app(self, client: GboxClient) -> None:
        with client.v1.boxes.android.with_streaming_response.list_app(
            "c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            android = response.parse()
            assert_matches_type(AndroidListAppResponse, android, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_list_app(self, client: GboxClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `box_id` but received ''"):
            client.v1.boxes.android.with_raw_response.list_app(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_pkg(self, client: GboxClient) -> None:
        android = client.v1.boxes.android.list_pkg(
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        )
        assert_matches_type(AndroidListPkgResponse, android, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_pkg_with_all_params(self, client: GboxClient) -> None:
        android = client.v1.boxes.android.list_pkg(
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
            pkg_type=["thirdParty"],
            running_filter=["running", "notRunning"],
        )
        assert_matches_type(AndroidListPkgResponse, android, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list_pkg(self, client: GboxClient) -> None:
        response = client.v1.boxes.android.with_raw_response.list_pkg(
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        android = response.parse()
        assert_matches_type(AndroidListPkgResponse, android, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list_pkg(self, client: GboxClient) -> None:
        with client.v1.boxes.android.with_streaming_response.list_pkg(
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            android = response.parse()
            assert_matches_type(AndroidListPkgResponse, android, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_list_pkg(self, client: GboxClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `box_id` but received ''"):
            client.v1.boxes.android.with_raw_response.list_pkg(
                box_id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_pkg_simple(self, client: GboxClient) -> None:
        android = client.v1.boxes.android.list_pkg_simple(
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        )
        assert_matches_type(AndroidListPkgSimpleResponse, android, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_pkg_simple_with_all_params(self, client: GboxClient) -> None:
        android = client.v1.boxes.android.list_pkg_simple(
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
            pkg_type=["thirdParty"],
        )
        assert_matches_type(AndroidListPkgSimpleResponse, android, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list_pkg_simple(self, client: GboxClient) -> None:
        response = client.v1.boxes.android.with_raw_response.list_pkg_simple(
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        android = response.parse()
        assert_matches_type(AndroidListPkgSimpleResponse, android, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list_pkg_simple(self, client: GboxClient) -> None:
        with client.v1.boxes.android.with_streaming_response.list_pkg_simple(
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            android = response.parse()
            assert_matches_type(AndroidListPkgSimpleResponse, android, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_list_pkg_simple(self, client: GboxClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `box_id` but received ''"):
            client.v1.boxes.android.with_raw_response.list_pkg_simple(
                box_id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_open(self, client: GboxClient) -> None:
        android = client.v1.boxes.android.open(
            package_name="com.example.myapp",
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        )
        assert android is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_open_with_all_params(self, client: GboxClient) -> None:
        android = client.v1.boxes.android.open(
            package_name="com.example.myapp",
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
            activity_name="com.android.settings.Settings",
        )
        assert android is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_open(self, client: GboxClient) -> None:
        response = client.v1.boxes.android.with_raw_response.open(
            package_name="com.example.myapp",
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        android = response.parse()
        assert android is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_open(self, client: GboxClient) -> None:
        with client.v1.boxes.android.with_streaming_response.open(
            package_name="com.example.myapp",
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            android = response.parse()
            assert android is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_open(self, client: GboxClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `box_id` but received ''"):
            client.v1.boxes.android.with_raw_response.open(
                package_name="com.example.myapp",
                box_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `package_name` but received ''"):
            client.v1.boxes.android.with_raw_response.open(
                package_name="",
                box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_restart(self, client: GboxClient) -> None:
        android = client.v1.boxes.android.restart(
            package_name="com.example.myapp",
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        )
        assert android is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_restart_with_all_params(self, client: GboxClient) -> None:
        android = client.v1.boxes.android.restart(
            package_name="com.example.myapp",
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
            activity_name="com.android.settings.Settings",
        )
        assert android is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_restart(self, client: GboxClient) -> None:
        response = client.v1.boxes.android.with_raw_response.restart(
            package_name="com.example.myapp",
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        android = response.parse()
        assert android is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_restart(self, client: GboxClient) -> None:
        with client.v1.boxes.android.with_streaming_response.restart(
            package_name="com.example.myapp",
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            android = response.parse()
            assert android is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_restart(self, client: GboxClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `box_id` but received ''"):
            client.v1.boxes.android.with_raw_response.restart(
                package_name="com.example.myapp",
                box_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `package_name` but received ''"):
            client.v1.boxes.android.with_raw_response.restart(
                package_name="",
                box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_restore(self, client: GboxClient) -> None:
        android = client.v1.boxes.android.restore(
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
            backup=b"raw file contents",
        )
        assert android is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_restore(self, client: GboxClient) -> None:
        response = client.v1.boxes.android.with_raw_response.restore(
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
            backup=b"raw file contents",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        android = response.parse()
        assert android is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_restore(self, client: GboxClient) -> None:
        with client.v1.boxes.android.with_streaming_response.restore(
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
            backup=b"raw file contents",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            android = response.parse()
            assert android is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_restore(self, client: GboxClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `box_id` but received ''"):
            client.v1.boxes.android.with_raw_response.restore(
                box_id="",
                backup=b"raw file contents",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_uninstall(self, client: GboxClient) -> None:
        android = client.v1.boxes.android.uninstall(
            package_name="com.example.myapp",
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        )
        assert android is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_uninstall_with_all_params(self, client: GboxClient) -> None:
        android = client.v1.boxes.android.uninstall(
            package_name="com.example.myapp",
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
            keep_data=True,
        )
        assert android is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_uninstall(self, client: GboxClient) -> None:
        response = client.v1.boxes.android.with_raw_response.uninstall(
            package_name="com.example.myapp",
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        android = response.parse()
        assert android is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_uninstall(self, client: GboxClient) -> None:
        with client.v1.boxes.android.with_streaming_response.uninstall(
            package_name="com.example.myapp",
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            android = response.parse()
            assert android is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_uninstall(self, client: GboxClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `box_id` but received ''"):
            client.v1.boxes.android.with_raw_response.uninstall(
                package_name="com.example.myapp",
                box_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `package_name` but received ''"):
            client.v1.boxes.android.with_raw_response.uninstall(
                package_name="",
                box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
            )


class TestAsyncAndroid:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    async def test_method_backup(self, async_client: AsyncGboxClient, respx_mock: MockRouter) -> None:
        respx_mock.post("/boxes/c9bdc193-b54b-4ddb-a035-5ac0c598d32d/android/packages/com.example.myapp/backup").mock(
            return_value=httpx.Response(200, json={"foo": "bar"})
        )
        android = await async_client.v1.boxes.android.backup(
            package_name="com.example.myapp",
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        )
        assert android.is_closed
        assert await android.json() == {"foo": "bar"}
        assert cast(Any, android.is_closed) is True
        assert isinstance(android, AsyncBinaryAPIResponse)

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    async def test_raw_response_backup(self, async_client: AsyncGboxClient, respx_mock: MockRouter) -> None:
        respx_mock.post("/boxes/c9bdc193-b54b-4ddb-a035-5ac0c598d32d/android/packages/com.example.myapp/backup").mock(
            return_value=httpx.Response(200, json={"foo": "bar"})
        )

        android = await async_client.v1.boxes.android.with_raw_response.backup(
            package_name="com.example.myapp",
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        )

        assert android.is_closed is True
        assert android.http_request.headers.get("X-Stainless-Lang") == "python"
        assert await android.json() == {"foo": "bar"}
        assert isinstance(android, AsyncBinaryAPIResponse)

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    async def test_streaming_response_backup(self, async_client: AsyncGboxClient, respx_mock: MockRouter) -> None:
        respx_mock.post("/boxes/c9bdc193-b54b-4ddb-a035-5ac0c598d32d/android/packages/com.example.myapp/backup").mock(
            return_value=httpx.Response(200, json={"foo": "bar"})
        )
        async with async_client.v1.boxes.android.with_streaming_response.backup(
            package_name="com.example.myapp",
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        ) as android:
            assert not android.is_closed
            assert android.http_request.headers.get("X-Stainless-Lang") == "python"

            assert await android.json() == {"foo": "bar"}
            assert cast(Any, android.is_closed) is True
            assert isinstance(android, AsyncStreamedBinaryAPIResponse)

        assert cast(Any, android.is_closed) is True

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    async def test_path_params_backup(self, async_client: AsyncGboxClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `box_id` but received ''"):
            await async_client.v1.boxes.android.with_raw_response.backup(
                package_name="com.example.myapp",
                box_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `package_name` but received ''"):
            await async_client.v1.boxes.android.with_raw_response.backup(
                package_name="",
                box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
            )

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    async def test_method_backup_all(self, async_client: AsyncGboxClient, respx_mock: MockRouter) -> None:
        respx_mock.post("/boxes/c9bdc193-b54b-4ddb-a035-5ac0c598d32d/android/packages/backup-all").mock(
            return_value=httpx.Response(200, json={"foo": "bar"})
        )
        android = await async_client.v1.boxes.android.backup_all(
            "c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        )
        assert android.is_closed
        assert await android.json() == {"foo": "bar"}
        assert cast(Any, android.is_closed) is True
        assert isinstance(android, AsyncBinaryAPIResponse)

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    async def test_raw_response_backup_all(self, async_client: AsyncGboxClient, respx_mock: MockRouter) -> None:
        respx_mock.post("/boxes/c9bdc193-b54b-4ddb-a035-5ac0c598d32d/android/packages/backup-all").mock(
            return_value=httpx.Response(200, json={"foo": "bar"})
        )

        android = await async_client.v1.boxes.android.with_raw_response.backup_all(
            "c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        )

        assert android.is_closed is True
        assert android.http_request.headers.get("X-Stainless-Lang") == "python"
        assert await android.json() == {"foo": "bar"}
        assert isinstance(android, AsyncBinaryAPIResponse)

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    async def test_streaming_response_backup_all(self, async_client: AsyncGboxClient, respx_mock: MockRouter) -> None:
        respx_mock.post("/boxes/c9bdc193-b54b-4ddb-a035-5ac0c598d32d/android/packages/backup-all").mock(
            return_value=httpx.Response(200, json={"foo": "bar"})
        )
        async with async_client.v1.boxes.android.with_streaming_response.backup_all(
            "c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        ) as android:
            assert not android.is_closed
            assert android.http_request.headers.get("X-Stainless-Lang") == "python"

            assert await android.json() == {"foo": "bar"}
            assert cast(Any, android.is_closed) is True
            assert isinstance(android, AsyncStreamedBinaryAPIResponse)

        assert cast(Any, android.is_closed) is True

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    async def test_path_params_backup_all(self, async_client: AsyncGboxClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `box_id` but received ''"):
            await async_client.v1.boxes.android.with_raw_response.backup_all(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_close(self, async_client: AsyncGboxClient) -> None:
        android = await async_client.v1.boxes.android.close(
            package_name="com.example.myapp",
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        )
        assert android is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_close(self, async_client: AsyncGboxClient) -> None:
        response = await async_client.v1.boxes.android.with_raw_response.close(
            package_name="com.example.myapp",
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        android = await response.parse()
        assert android is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_close(self, async_client: AsyncGboxClient) -> None:
        async with async_client.v1.boxes.android.with_streaming_response.close(
            package_name="com.example.myapp",
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            android = await response.parse()
            assert android is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_close(self, async_client: AsyncGboxClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `box_id` but received ''"):
            await async_client.v1.boxes.android.with_raw_response.close(
                package_name="com.example.myapp",
                box_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `package_name` but received ''"):
            await async_client.v1.boxes.android.with_raw_response.close(
                package_name="",
                box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_close_all(self, async_client: AsyncGboxClient) -> None:
        android = await async_client.v1.boxes.android.close_all(
            "c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        )
        assert android is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_close_all(self, async_client: AsyncGboxClient) -> None:
        response = await async_client.v1.boxes.android.with_raw_response.close_all(
            "c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        android = await response.parse()
        assert android is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_close_all(self, async_client: AsyncGboxClient) -> None:
        async with async_client.v1.boxes.android.with_streaming_response.close_all(
            "c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            android = await response.parse()
            assert android is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_close_all(self, async_client: AsyncGboxClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `box_id` but received ''"):
            await async_client.v1.boxes.android.with_raw_response.close_all(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_get(self, async_client: AsyncGboxClient) -> None:
        android = await async_client.v1.boxes.android.get(
            package_name="com.example.myapp",
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        )
        assert_matches_type(AndroidPkg, android, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_get(self, async_client: AsyncGboxClient) -> None:
        response = await async_client.v1.boxes.android.with_raw_response.get(
            package_name="com.example.myapp",
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        android = await response.parse()
        assert_matches_type(AndroidPkg, android, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_get(self, async_client: AsyncGboxClient) -> None:
        async with async_client.v1.boxes.android.with_streaming_response.get(
            package_name="com.example.myapp",
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            android = await response.parse()
            assert_matches_type(AndroidPkg, android, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_get(self, async_client: AsyncGboxClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `box_id` but received ''"):
            await async_client.v1.boxes.android.with_raw_response.get(
                package_name="com.example.myapp",
                box_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `package_name` but received ''"):
            await async_client.v1.boxes.android.with_raw_response.get(
                package_name="",
                box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_get_app(self, async_client: AsyncGboxClient) -> None:
        android = await async_client.v1.boxes.android.get_app(
            package_name="com.example.myapp",
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        )
        assert_matches_type(AndroidApp, android, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_get_app(self, async_client: AsyncGboxClient) -> None:
        response = await async_client.v1.boxes.android.with_raw_response.get_app(
            package_name="com.example.myapp",
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        android = await response.parse()
        assert_matches_type(AndroidApp, android, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_get_app(self, async_client: AsyncGboxClient) -> None:
        async with async_client.v1.boxes.android.with_streaming_response.get_app(
            package_name="com.example.myapp",
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            android = await response.parse()
            assert_matches_type(AndroidApp, android, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_get_app(self, async_client: AsyncGboxClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `box_id` but received ''"):
            await async_client.v1.boxes.android.with_raw_response.get_app(
                package_name="com.example.myapp",
                box_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `package_name` but received ''"):
            await async_client.v1.boxes.android.with_raw_response.get_app(
                package_name="",
                box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_get_connect_address(self, async_client: AsyncGboxClient) -> None:
        android = await async_client.v1.boxes.android.get_connect_address(
            "c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        )
        assert_matches_type(AndroidGetConnectAddressResponse, android, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_get_connect_address(self, async_client: AsyncGboxClient) -> None:
        response = await async_client.v1.boxes.android.with_raw_response.get_connect_address(
            "c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        android = await response.parse()
        assert_matches_type(AndroidGetConnectAddressResponse, android, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_get_connect_address(self, async_client: AsyncGboxClient) -> None:
        async with async_client.v1.boxes.android.with_streaming_response.get_connect_address(
            "c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            android = await response.parse()
            assert_matches_type(AndroidGetConnectAddressResponse, android, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_get_connect_address(self, async_client: AsyncGboxClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `box_id` but received ''"):
            await async_client.v1.boxes.android.with_raw_response.get_connect_address(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_install_overload_1(self, async_client: AsyncGboxClient) -> None:
        android = await async_client.v1.boxes.android.install(
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
            apk=b"raw file contents",
        )
        assert_matches_type(AndroidInstallResponse, android, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_install_with_all_params_overload_1(self, async_client: AsyncGboxClient) -> None:
        android = await async_client.v1.boxes.android.install(
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
            apk=b"raw file contents",
            open=False,
        )
        assert_matches_type(AndroidInstallResponse, android, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_install_overload_1(self, async_client: AsyncGboxClient) -> None:
        response = await async_client.v1.boxes.android.with_raw_response.install(
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
            apk=b"raw file contents",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        android = await response.parse()
        assert_matches_type(AndroidInstallResponse, android, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_install_overload_1(self, async_client: AsyncGboxClient) -> None:
        async with async_client.v1.boxes.android.with_streaming_response.install(
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
            apk=b"raw file contents",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            android = await response.parse()
            assert_matches_type(AndroidInstallResponse, android, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_install_overload_1(self, async_client: AsyncGboxClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `box_id` but received ''"):
            await async_client.v1.boxes.android.with_raw_response.install(
                box_id="",
                apk=b"raw file contents",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_install_overload_2(self, async_client: AsyncGboxClient) -> None:
        android = await async_client.v1.boxes.android.install(
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
            apk="https://example.com/app.apk",
        )
        assert_matches_type(AndroidInstallResponse, android, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_install_with_all_params_overload_2(self, async_client: AsyncGboxClient) -> None:
        android = await async_client.v1.boxes.android.install(
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
            apk="https://example.com/app.apk",
            open=False,
        )
        assert_matches_type(AndroidInstallResponse, android, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_install_overload_2(self, async_client: AsyncGboxClient) -> None:
        response = await async_client.v1.boxes.android.with_raw_response.install(
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
            apk="https://example.com/app.apk",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        android = await response.parse()
        assert_matches_type(AndroidInstallResponse, android, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_install_overload_2(self, async_client: AsyncGboxClient) -> None:
        async with async_client.v1.boxes.android.with_streaming_response.install(
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
            apk="https://example.com/app.apk",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            android = await response.parse()
            assert_matches_type(AndroidInstallResponse, android, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_install_overload_2(self, async_client: AsyncGboxClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `box_id` but received ''"):
            await async_client.v1.boxes.android.with_raw_response.install(
                box_id="",
                apk="https://example.com/app.apk",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_activities(self, async_client: AsyncGboxClient) -> None:
        android = await async_client.v1.boxes.android.list_activities(
            package_name="com.example.myapp",
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        )
        assert_matches_type(AndroidListActivitiesResponse, android, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list_activities(self, async_client: AsyncGboxClient) -> None:
        response = await async_client.v1.boxes.android.with_raw_response.list_activities(
            package_name="com.example.myapp",
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        android = await response.parse()
        assert_matches_type(AndroidListActivitiesResponse, android, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list_activities(self, async_client: AsyncGboxClient) -> None:
        async with async_client.v1.boxes.android.with_streaming_response.list_activities(
            package_name="com.example.myapp",
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            android = await response.parse()
            assert_matches_type(AndroidListActivitiesResponse, android, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_list_activities(self, async_client: AsyncGboxClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `box_id` but received ''"):
            await async_client.v1.boxes.android.with_raw_response.list_activities(
                package_name="com.example.myapp",
                box_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `package_name` but received ''"):
            await async_client.v1.boxes.android.with_raw_response.list_activities(
                package_name="",
                box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_app(self, async_client: AsyncGboxClient) -> None:
        android = await async_client.v1.boxes.android.list_app(
            "c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        )
        assert_matches_type(AndroidListAppResponse, android, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list_app(self, async_client: AsyncGboxClient) -> None:
        response = await async_client.v1.boxes.android.with_raw_response.list_app(
            "c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        android = await response.parse()
        assert_matches_type(AndroidListAppResponse, android, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list_app(self, async_client: AsyncGboxClient) -> None:
        async with async_client.v1.boxes.android.with_streaming_response.list_app(
            "c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            android = await response.parse()
            assert_matches_type(AndroidListAppResponse, android, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_list_app(self, async_client: AsyncGboxClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `box_id` but received ''"):
            await async_client.v1.boxes.android.with_raw_response.list_app(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_pkg(self, async_client: AsyncGboxClient) -> None:
        android = await async_client.v1.boxes.android.list_pkg(
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        )
        assert_matches_type(AndroidListPkgResponse, android, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_pkg_with_all_params(self, async_client: AsyncGboxClient) -> None:
        android = await async_client.v1.boxes.android.list_pkg(
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
            pkg_type=["thirdParty"],
            running_filter=["running", "notRunning"],
        )
        assert_matches_type(AndroidListPkgResponse, android, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list_pkg(self, async_client: AsyncGboxClient) -> None:
        response = await async_client.v1.boxes.android.with_raw_response.list_pkg(
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        android = await response.parse()
        assert_matches_type(AndroidListPkgResponse, android, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list_pkg(self, async_client: AsyncGboxClient) -> None:
        async with async_client.v1.boxes.android.with_streaming_response.list_pkg(
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            android = await response.parse()
            assert_matches_type(AndroidListPkgResponse, android, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_list_pkg(self, async_client: AsyncGboxClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `box_id` but received ''"):
            await async_client.v1.boxes.android.with_raw_response.list_pkg(
                box_id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_pkg_simple(self, async_client: AsyncGboxClient) -> None:
        android = await async_client.v1.boxes.android.list_pkg_simple(
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        )
        assert_matches_type(AndroidListPkgSimpleResponse, android, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_pkg_simple_with_all_params(self, async_client: AsyncGboxClient) -> None:
        android = await async_client.v1.boxes.android.list_pkg_simple(
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
            pkg_type=["thirdParty"],
        )
        assert_matches_type(AndroidListPkgSimpleResponse, android, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list_pkg_simple(self, async_client: AsyncGboxClient) -> None:
        response = await async_client.v1.boxes.android.with_raw_response.list_pkg_simple(
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        android = await response.parse()
        assert_matches_type(AndroidListPkgSimpleResponse, android, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list_pkg_simple(self, async_client: AsyncGboxClient) -> None:
        async with async_client.v1.boxes.android.with_streaming_response.list_pkg_simple(
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            android = await response.parse()
            assert_matches_type(AndroidListPkgSimpleResponse, android, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_list_pkg_simple(self, async_client: AsyncGboxClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `box_id` but received ''"):
            await async_client.v1.boxes.android.with_raw_response.list_pkg_simple(
                box_id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_open(self, async_client: AsyncGboxClient) -> None:
        android = await async_client.v1.boxes.android.open(
            package_name="com.example.myapp",
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        )
        assert android is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_open_with_all_params(self, async_client: AsyncGboxClient) -> None:
        android = await async_client.v1.boxes.android.open(
            package_name="com.example.myapp",
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
            activity_name="com.android.settings.Settings",
        )
        assert android is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_open(self, async_client: AsyncGboxClient) -> None:
        response = await async_client.v1.boxes.android.with_raw_response.open(
            package_name="com.example.myapp",
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        android = await response.parse()
        assert android is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_open(self, async_client: AsyncGboxClient) -> None:
        async with async_client.v1.boxes.android.with_streaming_response.open(
            package_name="com.example.myapp",
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            android = await response.parse()
            assert android is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_open(self, async_client: AsyncGboxClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `box_id` but received ''"):
            await async_client.v1.boxes.android.with_raw_response.open(
                package_name="com.example.myapp",
                box_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `package_name` but received ''"):
            await async_client.v1.boxes.android.with_raw_response.open(
                package_name="",
                box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_restart(self, async_client: AsyncGboxClient) -> None:
        android = await async_client.v1.boxes.android.restart(
            package_name="com.example.myapp",
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        )
        assert android is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_restart_with_all_params(self, async_client: AsyncGboxClient) -> None:
        android = await async_client.v1.boxes.android.restart(
            package_name="com.example.myapp",
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
            activity_name="com.android.settings.Settings",
        )
        assert android is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_restart(self, async_client: AsyncGboxClient) -> None:
        response = await async_client.v1.boxes.android.with_raw_response.restart(
            package_name="com.example.myapp",
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        android = await response.parse()
        assert android is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_restart(self, async_client: AsyncGboxClient) -> None:
        async with async_client.v1.boxes.android.with_streaming_response.restart(
            package_name="com.example.myapp",
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            android = await response.parse()
            assert android is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_restart(self, async_client: AsyncGboxClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `box_id` but received ''"):
            await async_client.v1.boxes.android.with_raw_response.restart(
                package_name="com.example.myapp",
                box_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `package_name` but received ''"):
            await async_client.v1.boxes.android.with_raw_response.restart(
                package_name="",
                box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_restore(self, async_client: AsyncGboxClient) -> None:
        android = await async_client.v1.boxes.android.restore(
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
            backup=b"raw file contents",
        )
        assert android is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_restore(self, async_client: AsyncGboxClient) -> None:
        response = await async_client.v1.boxes.android.with_raw_response.restore(
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
            backup=b"raw file contents",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        android = await response.parse()
        assert android is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_restore(self, async_client: AsyncGboxClient) -> None:
        async with async_client.v1.boxes.android.with_streaming_response.restore(
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
            backup=b"raw file contents",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            android = await response.parse()
            assert android is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_restore(self, async_client: AsyncGboxClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `box_id` but received ''"):
            await async_client.v1.boxes.android.with_raw_response.restore(
                box_id="",
                backup=b"raw file contents",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_uninstall(self, async_client: AsyncGboxClient) -> None:
        android = await async_client.v1.boxes.android.uninstall(
            package_name="com.example.myapp",
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        )
        assert android is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_uninstall_with_all_params(self, async_client: AsyncGboxClient) -> None:
        android = await async_client.v1.boxes.android.uninstall(
            package_name="com.example.myapp",
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
            keep_data=True,
        )
        assert android is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_uninstall(self, async_client: AsyncGboxClient) -> None:
        response = await async_client.v1.boxes.android.with_raw_response.uninstall(
            package_name="com.example.myapp",
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        android = await response.parse()
        assert android is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_uninstall(self, async_client: AsyncGboxClient) -> None:
        async with async_client.v1.boxes.android.with_streaming_response.uninstall(
            package_name="com.example.myapp",
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            android = await response.parse()
            assert android is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_uninstall(self, async_client: AsyncGboxClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `box_id` but received ''"):
            await async_client.v1.boxes.android.with_raw_response.uninstall(
                package_name="com.example.myapp",
                box_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `package_name` but received ''"):
            await async_client.v1.boxes.android.with_raw_response.uninstall(
                package_name="",
                box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Literal, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["BoxListParams"]


class BoxListParams(TypedDict, total=False):
    device_type: Annotated[str, PropertyInfo(alias="deviceType")]
    """Filter boxes by their device type (virtual, physical)"""

    labels: object
    """Filter boxes by their labels.

    Labels are key-value pairs that help identify and categorize boxes. Use this to
    filter boxes that match specific label criteria. For example, you can filter by
    project, environment, team, or any custom labels you've added to your boxes.
    """

    page: int
    """Page number"""

    page_size: Annotated[int, PropertyInfo(alias="pageSize")]
    """Page size"""

    status: List[Literal["all", "pending", "running", "error", "terminated"]]
    """
    Filter boxes by their current status (pending, running, stopped, error,
    terminated, all). Must be an array of statuses. Use 'all' to get boxes with any
    status.
    """

    type: List[Literal["all", "linux", "android"]]
    """Filter boxes by their type (linux, android, all).

    Must be an array of types. Use 'all' to get boxes of any type.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from typing_extensions import Required, TypeAlias, TypedDict

from ...._types import FileTypes

__all__ = ["AndroidInstallParams", "InstallAndroidPkgByFile", "InstallAndroidPkgByURL"]


class InstallAndroidPkgByFile(TypedDict, total=False):
    apk: Required[FileTypes]
    """APK file or ZIP archive to install (max file size: 512MB).

    **Single APK mode:**

    - Upload a single APK file (e.g., app.apk)
    - System will automatically detect and install as single APK

    **Multi-APK mode (automatically detected):**

    - Upload a ZIP archive containing multiple APK files
    - System will automatically detect ZIP format and install all APKs inside
    - ZIP filename example: com.reddit.frontpage-gplay.zip
    - ZIP contents example:

    com.reddit.frontpage-gplay.zip └── com.reddit.frontpage-gplay/ (folder) ├──
    reddit-base.apk (base APK) ├── reddit-arm64.apk (architecture-specific) ├──
    reddit-en.apk (language pack) └── reddit-mdpi.apk (density-specific resources)

    This is commonly used for split APKs where different components are separated by
    architecture, language, or screen density.
    """

    open: bool
    """Whether to open the app after installation.

    Will find and launch the launcher activity of the installed app. If there are
    multiple launcher activities, only one will be opened. If the installed APK has
    no launcher activity, this parameter will have no effect.
    """


class InstallAndroidPkgByURL(TypedDict, total=False):
    apk: Required[str]
    """HTTP URL to download APK file or ZIP archive (max file size: 512MB).

    **Single APK mode (automatically detected):**

    - Provide URL to a single APK file
    - System will automatically detect .apk extension and install as single APK
    - Example: https://example.com/app.apk

    **Multi-APK mode (automatically detected):**

    - Provide URL to a ZIP archive containing multiple APK files
    - System will automatically detect .zip extension and install all APKs inside
    - ZIP filename example: com.reddit.frontpage-gplay.zip
    - ZIP contents example:

    com.reddit.frontpage-gplay.zip └── com.reddit.frontpage-gplay/ (folder) ├──
    reddit-base.apk (base APK) ├── reddit-arm64.apk (architecture-specific) ├──
    reddit-en.apk (language pack) └── reddit-mdpi.apk (density-specific resources)

    - Example URL: https://example.com/com.reddit.frontpage-gplay.zip

    This is commonly used for split APKs where different components are separated by
    architecture, language, or screen density.
    """

    open: bool
    """Whether to open the app after installation.

    Will find and launch the launcher activity of the installed app. If there are
    multiple launcher activities, only one will be opened. If the installed APK has
    no launcher activity, this parameter will have no effect.
    """


AndroidInstallParams: TypeAlias = Union[InstallAndroidPkgByFile, InstallAndroidPkgByURL]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .dir import Dir as Dir
from .file import File as File
from .android_app import AndroidApp as AndroidApp
from .android_pkg import AndroidPkg as AndroidPkg
from .media_album import MediaAlbum as MediaAlbum
from .media_photo import MediaPhoto as MediaPhoto
from .media_video import MediaVideo as MediaVideo
from .action_result import ActionResult as ActionResult
from .f_info_params import FInfoParams as FInfoParams
from .f_list_params import FListParams as FListParams
from .f_read_params import FReadParams as FReadParams
from .f_write_params import FWriteParams as FWriteParams
from .f_exists_params import FExistsParams as FExistsParams
from .f_info_response import FInfoResponse as FInfoResponse
from .f_list_response import FListResponse as FListResponse
from .f_read_response import FReadResponse as FReadResponse
from .f_remove_params import FRemoveParams as FRemoveParams
from .f_rename_params import FRenameParams as FRenameParams
from .action_ai_params import ActionAIParams as ActionAIParams
from .proxy_set_params import ProxySetParams as ProxySetParams
from .action_tap_params import ActionTapParams as ActionTapParams
from .f_exists_response import FExistsResponse as FExistsResponse
from .f_remove_response import FRemoveResponse as FRemoveResponse
from .f_rename_response import FRenameResponse as FRenameResponse
from .action_ai_response import ActionAIResponse as ActionAIResponse
from .action_drag_params import ActionDragParams as ActionDragParams
from .action_move_params import ActionMoveParams as ActionMoveParams
from .action_type_params import ActionTypeParams as ActionTypeParams
from .proxy_get_response import ProxyGetResponse as ProxyGetResponse
from .proxy_set_response import ProxySetResponse as ProxySetResponse
from .action_click_params import ActionClickParams as ActionClickParams
from .action_swipe_params import ActionSwipeParams as ActionSwipeParams
from .action_touch_params import ActionTouchParams as ActionTouchParams
from .android_open_params import AndroidOpenParams as AndroidOpenParams
from .action_scroll_params import ActionScrollParams as ActionScrollParams
from .action_common_options import ActionCommonOptions as ActionCommonOptions
from .action_extract_params import ActionExtractParams as ActionExtractParams
from .android_install_params import AndroidInstallParams as AndroidInstallParams
from .android_restart_params import AndroidRestartParams as AndroidRestartParams
from .android_restore_params import AndroidRestoreParams as AndroidRestoreParams
from .browser_cdp_url_params import BrowserCdpURLParams as BrowserCdpURLParams
from .action_extract_response import ActionExtractResponse as ActionExtractResponse
from .action_press_key_params import ActionPressKeyParams as ActionPressKeyParams
from .android_list_pkg_params import AndroidListPkgParams as AndroidListPkgParams
from .browser_open_tab_params import BrowserOpenTabParams as BrowserOpenTabParams
from .action_long_press_params import ActionLongPressParams as ActionLongPressParams
from .action_screenshot_params import ActionScreenshotParams as ActionScreenshotParams
from .action_settings_response import ActionSettingsResponse as ActionSettingsResponse
from .android_install_response import AndroidInstallResponse as AndroidInstallResponse
from .android_uninstall_params import AndroidUninstallParams as AndroidUninstallParams
from .browser_cdp_url_response import BrowserCdpURLResponse as BrowserCdpURLResponse
from .media_get_media_response import MediaGetMediaResponse as MediaGetMediaResponse
from .action_screenshot_options import ActionScreenshotOptions as ActionScreenshotOptions
from .android_list_app_response import AndroidListAppResponse as AndroidListAppResponse
from .android_list_pkg_response import AndroidListPkgResponse as AndroidListPkgResponse
from .browser_get_tabs_response import BrowserGetTabsResponse as BrowserGetTabsResponse
from .browser_open_tab_response import BrowserOpenTabResponse as BrowserOpenTabResponse
from .browser_update_tab_params import BrowserUpdateTabParams as BrowserUpdateTabParams
from .media_create_album_params import MediaCreateAlbumParams as MediaCreateAlbumParams
from .media_list_media_response import MediaListMediaResponse as MediaListMediaResponse
from .media_update_album_params import MediaUpdateAlbumParams as MediaUpdateAlbumParams
from .action_press_button_params import ActionPressButtonParams as ActionPressButtonParams
from .action_screenshot_response import ActionScreenshotResponse as ActionScreenshotResponse
from .browser_close_tab_response import BrowserCloseTabResponse as BrowserCloseTabResponse
from .media_list_albums_response import MediaListAlbumsResponse as MediaListAlbumsResponse
from .action_common_options_param import ActionCommonOptionsParam as ActionCommonOptionsParam
from .browser_switch_tab_response import BrowserSwitchTabResponse as BrowserSwitchTabResponse
from .browser_update_tab_response import BrowserUpdateTabResponse as BrowserUpdateTabResponse
from .action_rewind_extract_params import ActionRewindExtractParams as ActionRewindExtractParams
from .storage_presigned_url_params import StoragePresignedURLParams as StoragePresignedURLParams
from .action_recording_start_params import ActionRecordingStartParams as ActionRecordingStartParams
from .action_screen_layout_response import ActionScreenLayoutResponse as ActionScreenLayoutResponse
from .action_screen_rotation_params import ActionScreenRotationParams as ActionScreenRotationParams
from .action_settings_update_params import ActionSettingsUpdateParams as ActionSettingsUpdateParams
from .action_recording_stop_response import ActionRecordingStopResponse as ActionRecordingStopResponse
from .action_rewind_extract_response import ActionRewindExtractResponse as ActionRewindExtractResponse
from .action_settings_reset_response import ActionSettingsResetResponse as ActionSettingsResetResponse
from .android_list_pkg_simple_params import AndroidListPkgSimpleParams as AndroidListPkgSimpleParams
from .storage_presigned_url_response import StoragePresignedURLResponse as StoragePresignedURLResponse
from .action_screenshot_options_param import ActionScreenshotOptionsParam as ActionScreenshotOptionsParam
from .action_settings_update_response import ActionSettingsUpdateResponse as ActionSettingsUpdateResponse
from .android_list_activities_response import AndroidListActivitiesResponse as AndroidListActivitiesResponse
from .android_list_pkg_simple_response import AndroidListPkgSimpleResponse as AndroidListPkgSimpleResponse
from .media_get_media_support_response import MediaGetMediaSupportResponse as MediaGetMediaSupportResponse
from .android_get_connect_address_response import AndroidGetConnectAddressResponse as AndroidGetConnectAddressResponse
